@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2013-2021 Laboratoire de Recherche et Développement
+// Copyright (C) 2013-2023 Laboratoire de Recherche et Développement
 // de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -24,7 +24,7 @@
 #include <argp.h>
 #include "common_setup.hh"
 
-const char argp_program_doc[] ="\
+static const char argp_program_doc[] = "\
 Common fine-tuning options for programs installed with Spot.\n\
 \n\
 The argument of -x or --extra-options is a comma-separated list of KEY=INT \
@@ -47,9 +47,17 @@ depends on the --low, --medium, or --high settings.") },
     { DOC("tls-max-states",
           "Maximum number of states of automata involved in automata-based \
 implication checks for formula simplifications.  Defaults to 64.") },
+    { DOC("tls-max-ops",
+          "Maximum number of operands in n-ary opertors (or, and) on which \
+implication-based simplifications are attempted.  Defaults to 16.") },
     { nullptr, 0, nullptr, 0, "Translation options:", 0 },
     { DOC("ltl-split", "Set to 0 to disable the translation of automata \
 as product or sum of subformulas.") },
+    { DOC("branch-post", "Set to 0 to disable branching-postponement \
+(done during translation, may create more states) and delayed-branching \
+(almost similar, but done after translation to only remove states).  \
+Set to 1 to force branching-postponement, and to 2 \
+to force delayed-branching.  By default delayed-branching is used.") },
     { DOC("comp-susp", "Set to 1 to enable compositional suspension, \
 as described in our SPIN'13 paper (see Bibliography below).  Set to 2, \
 to build only the skeleton TGBA without composing it.  Set to 0 (the \
@@ -75,6 +83,9 @@ only if it is smaller than the original skeleton.  This option is only \
 used when comp-susp=1 and default to 1 or 2 depending on whether --small \
 or --deterministic is specified.") },
     { nullptr, 0, nullptr, 0, "Postprocessing options:", 0 },
+    { DOC("acd", "Set to 1 (the default) to use paritize automata using \
+the alternatinc cycle decomposition.  Set to 0 to use paritization based \
+on latest appearance record variants.") },
     { DOC("scc-filter", "Set to 1 (the default) to enable \
 SCC-pruning and acceptance simplification at the beginning of \
 post-processing.  Transitions that are outside of accepting SCC are \
@@ -164,6 +175,10 @@ Set to 1 to use only direct simulation.  Set to 2 to use only reverse \
 simulation.  Set to 3 to iterate both direct and reverse simulations.   \
 The default is the value of parameter \"simul\" in --high mode, and 0 \
 therwise.") },
+    { DOC("merge-states-min", "Number of states above which states are \
+merged using a cheap approximation of a bisimulation quotient before \
+attempting simulation-based reductions.   Defaults to 128.  Set to 0 to \
+never merge states.") },
     { DOC("simul-max", "Number of states above which simulation-based \
 reductions are skipped. Defaults to 4096.  Set to 0 to disable.  This \
 applies to all simulation-based optimization, including thoses of the \
