@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.7.5.  */
+/* A Bison parser, made by GNU Bison 3.8.2.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
@@ -16,7 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -46,10 +46,10 @@
    USER NAME SPACE" below.  */
 
 /* Identify Bison output, and Bison version.  */
-#define YYBISON 30705
+#define YYBISON 30802
 
 /* Bison version string.  */
-#define YYBISON_VERSION "3.7.5"
+#define YYBISON_VERSION "3.8.2"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -257,8 +257,10 @@ typedef int YYSTYPE;
 
 extern YYSTYPE yylval;
 
+
 int yyparse (void);
 
+
 #endif /* !YY_YY_PARSER_H_INCLUDED  */
 /* Symbol kind.  */
 enum yysymbol_kind_t
@@ -478,12 +480,18 @@ typedef int yy_state_fast_t;
 # define YY_USE(E) /* empty */
 #endif
 
-#if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
 /* Suppress an incorrect diagnostic about yylval being uninitialized.  */
-# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                            \
+#if defined __GNUC__ && ! defined __ICC && 406 <= __GNUC__ * 100 + __GNUC_MINOR__
+# if __GNUC__ * 100 + __GNUC_MINOR__ < 407
+#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
+    _Pragma ("GCC diagnostic push")                                     \
+    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")
+# else
+#  define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                           \
     _Pragma ("GCC diagnostic push")                                     \
     _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")              \
     _Pragma ("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
+# endif
 # define YY_IGNORE_MAYBE_UNINITIALIZED_END      \
     _Pragma ("GCC diagnostic pop")
 #else
@@ -701,7 +709,7 @@ static const yytype_int8 yytranslate[] =
 };
 
 #if YYDEBUG
-  /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
+/* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
        0,    96,    96,   102,   106,   110,   111,   118,   122,   123,
@@ -743,18 +751,6 @@ yysymbol_name (yysymbol_kind_t yysymbol)
 }
 #endif
 
-#ifdef YYPRINT
-/* YYTOKNUM[NUM] -- (External) token number corresponding to the
-   (internal) symbol number NUM (which must be that of a token).  */
-static const yytype_int16 yytoknum[] =
-{
-       0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
-     265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
-     275,   276,   277,   278,   279,   280,   281,   282,   283,   284,
-     285,   286,   287,   288,   289,   290,   291,   292
-};
-#endif
-
 #define YYPACT_NINF (-38)
 
 #define yypact_value_is_default(Yyn) \
@@ -765,8 +761,8 @@ static const yytype_int16 yytoknum[] =
 #define yytable_value_is_error(Yyn) \
   0
 
-  /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
-     STATE-NUM.  */
+/* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
+   STATE-NUM.  */
 static const yytype_int8 yypact[] =
 {
       -6,     7,    15,    14,    46,   -38,    49,    44,    50,   -38,
@@ -780,9 +776,9 @@ static const yytype_int8 yypact[] =
      -38,   -38,    10,    10
 };
 
-  /* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
-     Performed when YYTABLE does not specify something else to do.  Zero
-     means the default is an error.  */
+/* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
+   Performed when YYTABLE does not specify something else to do.  Zero
+   means the default is an error.  */
 static const yytype_int8 yydefact[] =
 {
        0,     0,     0,     0,     0,     1,     0,     0,     0,     6,
@@ -796,23 +792,23 @@ static const yytype_int8 yydefact[] =
       19,    18,    31,    32
 };
 
-  /* YYPGOTO[NTERM-NUM].  */
+/* YYPGOTO[NTERM-NUM].  */
 static const yytype_int8 yypgoto[] =
 {
      -38,   -38,   -38,   -38,   -38,   -38,   -38,    62,   -38,   -37,
      -38,    35,   -38,   -38,   -38,    52,   -38,   -38,   -38
 };
 
-  /* YYDEFGOTO[NTERM-NUM].  */
+/* YYDEFGOTO[NTERM-NUM].  */
 static const yytype_int8 yydefgoto[] =
 {
        0,     2,     3,     7,    10,    12,    24,    25,    26,    54,
       55,    61,    27,    28,    29,    42,    30,    31,    32
 };
 
-  /* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
-     positive, shift that token.  If negative, reduce the rule whose
-     number is the opposite.  If YYTABLE_NINF, syntax error.  */
+/* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
+   positive, shift that token.  If negative, reduce the rule whose
+   number is the opposite.  If YYTABLE_NINF, syntax error.  */
 static const yytype_int8 yytable[] =
 {
       47,    14,     1,    48,    49,    65,    66,    67,    68,    69,
@@ -839,8 +835,8 @@ static const yytype_int8 yycheck[] =
        3,     3,    26,     3,    26,     3,    24,    52,    36
 };
 
-  /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
-     symbol of state STATE-NUM.  */
+/* YYSTOS[STATE-NUM] -- The symbol kind of the accessing symbol of
+   state STATE-NUM.  */
 static const yytype_int8 yystos[] =
 {
        0,     8,    39,    40,     5,     0,     9,    41,     5,     3,
@@ -854,7 +850,7 @@ static const yytype_int8 yystos[] =
       47,    47,    47,    47
 };
 
-  /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
+/* YYR1[RULE-NUM] -- Symbol kind of the left-hand side of rule RULE-NUM.  */
 static const yytype_int8 yyr1[] =
 {
        0,    38,    39,    40,    41,    42,    42,    43,    44,    44,
@@ -864,7 +860,7 @@ static const yytype_int8 yyr1[] =
       53,    53,    53,    53,    54,    55,    56
 };
 
-  /* YYR2[YYN] -- Number of symbols on the right hand side of rule YYN.  */
+/* YYR2[RULE-NUM] -- Number of symbols on the right-hand side of rule RULE-NUM.  */
 static const yytype_int8 yyr2[] =
 {
        0,     2,     3,     4,     3,     2,     1,     2,     3,     2,
@@ -883,6 +879,7 @@ enum { YYENOMEM = -2 };
 #define YYACCEPT        goto yyacceptlab
 #define YYABORT         goto yyabortlab
 #define YYERROR         goto yyerrorlab
+#define YYNOMEM         goto yyexhaustedlab
 
 
 #define YYRECOVERING()  (!!yyerrstatus)
@@ -923,10 +920,7 @@ do {                                            \
     YYFPRINTF Args;                             \
 } while (0)
 
-/* This macro is provided for backward compatibility. */
-# ifndef YY_LOCATION_PRINT
-#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
-# endif
+
 
 
 # define YY_SYMBOL_PRINT(Title, Kind, Value, Location)                    \
@@ -953,10 +947,6 @@ yy_symbol_value_print (FILE *yyo,
   YY_USE (yyoutput);
   if (!yyvaluep)
     return;
-# ifdef YYPRINT
-  if (yykind < YYNTOKENS)
-    YYPRINT (yyo, yytoknum[yykind], *yyvaluep);
-# endif
   YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   YY_USE (yykind);
   YY_IGNORE_MAYBE_UNINITIALIZED_END
@@ -1141,6 +1131,7 @@ yyparse (void)
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yychar = YYEMPTY; /* Cause a token to be read.  */
+
   goto yysetstate;
 
 
@@ -1166,7 +1157,7 @@ yysetstate:
 
   if (yyss + yystacksize - 1 <= yyssp)
 #if !defined yyoverflow && !defined YYSTACK_RELOCATE
-    goto yyexhaustedlab;
+    YYNOMEM;
 #else
     {
       /* Get the current used size of the three stacks, in elements.  */
@@ -1194,7 +1185,7 @@ yysetstate:
 # else /* defined YYSTACK_RELOCATE */
       /* Extend the stack our own way.  */
       if (YYMAXDEPTH <= yystacksize)
-        goto yyexhaustedlab;
+        YYNOMEM;
       yystacksize *= 2;
       if (YYMAXDEPTH < yystacksize)
         yystacksize = YYMAXDEPTH;
@@ -1205,7 +1196,7 @@ yysetstate:
           YY_CAST (union yyalloc *,
                    YYSTACK_ALLOC (YY_CAST (YYSIZE_T, YYSTACK_BYTES (yystacksize))));
         if (! yyptr)
-          goto yyexhaustedlab;
+          YYNOMEM;
         YYSTACK_RELOCATE (yyss_alloc, yyss);
         YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
@@ -1227,6 +1218,7 @@ yysetstate:
     }
 #endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */
 
+
   if (yystate == YYFINAL)
     YYACCEPT;
 
@@ -1341,209 +1333,209 @@ yyreduce:
   case 3: /* initial: T_initial T_intval T_intval T_semi  */
 #line 102 "parser.y"
                                       { actInit(&yyvsp[-2],&yyvsp[-1]); }
-#line 1345 "parser.c"
+#line 1337 "parser.c"
     break;
 
   case 4: /* inputs: T_inputs inputSeq T_semi  */
 #line 106 "parser.y"
                             { actInputs(); }
-#line 1351 "parser.c"
+#line 1343 "parser.c"
     break;
 
   case 5: /* inputSeq: inputSeq T_id  */
 #line 110 "parser.y"
                  { actAddInput(&yyvsp[0]); }
-#line 1357 "parser.c"
+#line 1349 "parser.c"
     break;
 
   case 6: /* inputSeq: T_id  */
 #line 111 "parser.y"
                  { actAddInput(&yyvsp[0]); }
-#line 1363 "parser.c"
+#line 1355 "parser.c"
     break;
 
   case 17: /* assign: T_id T_equal expr  */
 #line 137 "parser.y"
                      { actAssign(&yyvsp[-2],&yyvsp[0]); }
-#line 1369 "parser.c"
+#line 1361 "parser.c"
     break;
 
   case 18: /* expr: expr T_and expr  */
 #line 141 "parser.y"
                         { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_and); }
-#line 1375 "parser.c"
+#line 1367 "parser.c"
     break;
 
   case 19: /* expr: expr T_nand expr  */
 #line 142 "parser.y"
                         { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_nand); }
-#line 1381 "parser.c"
+#line 1373 "parser.c"
     break;
 
   case 20: /* expr: expr T_xor expr  */
 #line 143 "parser.y"
                         { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_xor); }
-#line 1387 "parser.c"
+#line 1379 "parser.c"
     break;
 
   case 21: /* expr: expr T_or expr  */
 #line 144 "parser.y"
                         { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_or); }
-#line 1393 "parser.c"
+#line 1385 "parser.c"
     break;
 
   case 22: /* expr: expr T_nor expr  */
 #line 145 "parser.y"
                         { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_nor); }
-#line 1399 "parser.c"
+#line 1391 "parser.c"
     break;
 
   case 23: /* expr: expr T_imp expr  */
 #line 146 "parser.y"
                         { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_imp); }
-#line 1405 "parser.c"
+#line 1397 "parser.c"
     break;
 
   case 24: /* expr: expr T_biimp expr  */
 #line 147 "parser.y"
                         { actOpr2(&yyval,&yyvsp[-2],&yyvsp[0],bddop_biimp); }
-#line 1411 "parser.c"
+#line 1403 "parser.c"
     break;
 
   case 25: /* expr: T_not expr  */
 #line 148 "parser.y"
                         { actNot(&yyval,&yyvsp[0]); }
-#line 1417 "parser.c"
+#line 1409 "parser.c"
     break;
 
   case 26: /* expr: T_lpar expr T_rpar  */
 #line 149 "parser.y"
                         { yyval.bval = yyvsp[-1].bval; }
-#line 1423 "parser.c"
+#line 1415 "parser.c"
     break;
 
   case 27: /* expr: T_id  */
 #line 150 "parser.y"
                         { actId(&yyval,&yyvsp[0]); }
-#line 1429 "parser.c"
+#line 1421 "parser.c"
     break;
 
   case 28: /* expr: T_true  */
 #line 151 "parser.y"
                         { yyval.bval = new bdd(bddtrue); }
-#line 1435 "parser.c"
+#line 1427 "parser.c"
     break;
 
   case 29: /* expr: T_false  */
 #line 152 "parser.y"
                         { yyval.bval = new bdd(bddfalse); }
-#line 1441 "parser.c"
+#line 1433 "parser.c"
     break;
 
   case 30: /* expr: quantifier  */
 #line 153 "parser.y"
                         { yyval.bval = yyvsp[0].bval; }
-#line 1447 "parser.c"
+#line 1439 "parser.c"
     break;
 
   case 31: /* quantifier: T_exist varlist T_dot expr  */
 #line 157 "parser.y"
                               { actExist(&yyval,&yyvsp[-2],&yyvsp[0]); }
-#line 1453 "parser.c"
+#line 1445 "parser.c"
     break;
 
   case 32: /* quantifier: T_forall varlist T_dot expr  */
 #line 158 "parser.y"
                                  { actForall(&yyval,&yyvsp[-2],&yyvsp[0]); }
-#line 1459 "parser.c"
+#line 1451 "parser.c"
     break;
 
   case 33: /* varlist: varlist T_id  */
 #line 162 "parser.y"
                 { actQuantVar2(&yyval,&yyvsp[0],&yyvsp[-1]); }
-#line 1465 "parser.c"
+#line 1457 "parser.c"
     break;
 
   case 34: /* varlist: T_id  */
 #line 163 "parser.y"
                 { actQuantVar1(&yyval,&yyvsp[0]); }
-#line 1471 "parser.c"
+#line 1463 "parser.c"
     break;
 
   case 35: /* size: T_size T_id  */
 #line 168 "parser.y"
                { actSize(&yyvsp[0]); }
-#line 1477 "parser.c"
+#line 1469 "parser.c"
     break;
 
   case 36: /* dot: T_dumpdot T_str T_id  */
 #line 172 "parser.y"
                         { actDot(&yyvsp[-1],&yyvsp[0]); }
-#line 1483 "parser.c"
+#line 1475 "parser.c"
     break;
 
   case 37: /* reorder: T_reorder method  */
 #line 176 "parser.y"
                                    { bdd_reorder(yyvsp[0].ival); }
-#line 1489 "parser.c"
+#line 1481 "parser.c"
     break;
 
   case 38: /* reorder: T_autoreorder T_intval method  */
 #line 177 "parser.y"
                                    { actAutoreorder(&yyvsp[-1],&yyvsp[0]); }
-#line 1495 "parser.c"
+#line 1487 "parser.c"
     break;
 
   case 39: /* method: T_win2  */
 #line 181 "parser.y"
                 { yyval.ival = BDD_REORDER_WIN2; }
-#line 1501 "parser.c"
+#line 1493 "parser.c"
     break;
 
   case 40: /* method: T_win2ite  */
 #line 182 "parser.y"
                 { yyval.ival = BDD_REORDER_WIN2ITE; }
-#line 1507 "parser.c"
+#line 1499 "parser.c"
     break;
 
   case 41: /* method: T_sift  */
 #line 183 "parser.y"
                 { yyval.ival = BDD_REORDER_SIFT; }
-#line 1513 "parser.c"
+#line 1505 "parser.c"
     break;
 
   case 42: /* method: T_siftite  */
 #line 184 "parser.y"
                 { yyval.ival = BDD_REORDER_SIFTITE; }
-#line 1519 "parser.c"
+#line 1511 "parser.c"
     break;
 
   case 43: /* method: T_none  */
 #line 185 "parser.y"
                 { yyval.ival = BDD_REORDER_NONE; }
-#line 1525 "parser.c"
+#line 1517 "parser.c"
     break;
 
   case 44: /* cache: T_cache  */
 #line 189 "parser.y"
            { actCache(); }
-#line 1531 "parser.c"
+#line 1523 "parser.c"
     break;
 
   case 45: /* tautology: T_tautology T_id  */
 #line 193 "parser.y"
                     { actTautology(&yyvsp[0]); }
-#line 1537 "parser.c"
+#line 1529 "parser.c"
     break;
 
   case 46: /* print: T_print T_id  */
 #line 197 "parser.y"
                 { actPrint(&yyvsp[0]); }
-#line 1543 "parser.c"
+#line 1535 "parser.c"
     break;
 
 
-#line 1547 "parser.c"
+#line 1539 "parser.c"
 
       default: break;
     }
@@ -1625,6 +1617,7 @@ yyerrorlab:
      label yyerrorlab therefore never appears in user code.  */
   if (0)
     YYERROR;
+  ++yynerrs;
 
   /* Do not reclaim the symbols of the rule whose action triggered
      this YYERROR.  */
@@ -1685,7 +1678,7 @@ yyerrlab1:
 `-------------------------------------*/
 yyacceptlab:
   yyresult = 0;
-  goto yyreturn;
+  goto yyreturnlab;
 
 
 /*-----------------------------------.
@@ -1693,24 +1686,22 @@ yyacceptlab:
 `-----------------------------------*/
 yyabortlab:
   yyresult = 1;
-  goto yyreturn;
+  goto yyreturnlab;
 
 
-#if !defined yyoverflow
-/*-------------------------------------------------.
-| yyexhaustedlab -- memory exhaustion comes here.  |
-`-------------------------------------------------*/
+/*-----------------------------------------------------------.
+| yyexhaustedlab -- YYNOMEM (memory exhaustion) comes here.  |
+`-----------------------------------------------------------*/
 yyexhaustedlab:
   yyerror (YY_("memory exhausted"));
   yyresult = 2;
-  goto yyreturn;
-#endif
+  goto yyreturnlab;
 
 
-/*-------------------------------------------------------.
-| yyreturn -- parsing is finished, clean up and return.  |
-`-------------------------------------------------------*/
-yyreturn:
+/*----------------------------------------------------------.
+| yyreturnlab -- parsing is finished, clean up and return.  |
+`----------------------------------------------------------*/
+yyreturnlab:
   if (yychar != YYEMPTY)
     {
       /* Make sure we have latest lookahead translation.  See comments at
