@@ -1,5 +1,5 @@
 /*========================================================================
-	       Copyright (C) 1996-2002 by Jorn Lind-Nielsen
+	       Copyright (C) 1996-2002, 2021 by Jorn Lind-Nielsen
 			    All rights reserved
 
     Permission is hereby granted, without written agreement and without
@@ -45,7 +45,6 @@
 #include <assert.h>
 
 #include "kernel.h"
-#include "cache.h"
 #include "prime.h"
 
 /*************************************************************************
@@ -103,6 +102,8 @@ jmp_buf      bddexception;      /* Long-jump point for interrupting calc. */
 int          bddresized;        /* Flag indicating a resize of the nodetable */
 int          bddcachesize;      /* Size of the operator caches */
 int          bddhashsize;       /* Size of the BDD node hash */
+int*         bddrecstack;       /* Internal recursion stack */
+int*         bddrecstacktop;    /* Internal recursion stack top */
 
 bddCacheStat bddcachestats;
 
@@ -273,6 +274,7 @@ void bdd_done(void)
 
    free(bddnodes);
    free(bddrefstack);
+   free(bddrecstack);
    free(bddvarset);
    free(bddvar2level);
    free(bddlevel2var);
@@ -280,6 +282,7 @@ void bdd_done(void)
 
    bddnodes = NULL;
    bddrefstack = NULL;
+   bddrecstack = NULL;
    bddvarset = NULL;
 
    bdd_operator_done();
@@ -374,7 +377,11 @@ int bdd_setvarnum(int num)
 
    if (__likely(bddrefstack != NULL))
       free(bddrefstack);
-   bddrefstack = bddrefstacktop = (int*)malloc(sizeof(int)*(num*2+4));
+   bddrefstack = bddrefstacktop = (int*)malloc(sizeof(int)*((num+2)*2));
+
+   if (__likely(bddrecstack != NULL))
+      free(bddrecstack);
+   bddrecstack = bddrecstacktop = (int*)malloc(sizeof(int)*((num+1)*9));
 
    for(bdv=bddvarnum ; bddvarnum < num; bddvarnum++)
    {
@@ -1011,6 +1018,32 @@ BDD bdd_high(BDD root)
 }
 
 
+int bdd_stable_cmp(BDD left, BDD right)
+{
+  for (;;)
+    {
+      if (left == right || ISCONST(left) || ISCONST(right))
+        return left - right;
+      int vl = bddlevel2var[LEVEL(left)];
+      int vr = bddlevel2var[LEVEL(right)];
+      if (vl != vr)
+        return vr - vl;
+      // We check the high side before low, this way
+      //  !a&b comes before a&!b and a&b
+      BDD dl = HIGH(left);
+      BDD dr = HIGH(right);
+      if (dl != dr)
+        {
+          left = dl;
+          right = dr;
+        }
+      else
+        {
+          left = LOW(left);
+          right = LOW(right);
+        }
+    }
+}
 
 /*************************************************************************
   Garbage collection and node referencing
