@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2015, 2018-2019 Laboratoire de Recherche et
+// Copyright (C) 2015, 2018-2020 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -97,6 +97,7 @@ namespace spot
       case op::ap:
       case op::Not:
       case op::X:
+      case op::strong_X:
       case op::Closure:
       case op::NegClosure:
       case op::NegClosureMarked:
@@ -275,7 +276,9 @@ namespace spot
           break;
         }
       }
-    return entry.first->second = out;
+    // The recursion may have invalidated the "entry" iterator, so do
+    // not reuse it.
+    return cache_[in] = out;
   }
 
   formula unabbreviate(formula in, const char* opt)
