@@ -1067,81 +1067,6 @@ namespace spot
     return res;
   }
 
-  namespace
-  {
-    bool
-    has_parity_prefix_aux(acc_cond original,
-                          acc_cond &new_cond,
-                          std::vector<unsigned> &colors,
-                          std::vector<acc_cond::acc_code> elements,
-                          acc_cond::acc_op op)
-    {
-      if (elements.size() > 2)
-        {
-          new_cond = original;
-          return false;
-        }
-      if (elements.size() == 2)
-        {
-          unsigned pos = (elements[1].back().sub.op == op
-                          && elements[1][0].mark.is_singleton());
-          if (!(elements[0].back().sub.op == op || pos))
-            {
-              new_cond = original;
-              return false;
-            }
-          if ((elements[1 - pos].used_sets() & elements[pos][0].mark))
-            {
-              new_cond = original;
-              return false;
-            }
-          if (!elements[pos][0].mark.is_singleton())
-            {
-              return false;
-            }
-          colors.push_back(elements[pos][0].mark.min_set() - 1);
-          elements[1 - pos].has_parity_prefix(new_cond, colors);
-          return true;
-        }
-      return false;
-    }
-  }
-
-  bool
-  acc_cond::acc_code::has_parity_prefix(acc_cond &new_cond,
-                                        std::vector<unsigned> &colors) const
-  {
-    auto disj = top_disjuncts();
-    if (!(has_parity_prefix_aux((*this), new_cond, colors,
-            disj, acc_cond::acc_op::Inf) ||
-          has_parity_prefix_aux((*this), new_cond, colors,
-            top_conjuncts(), acc_cond::acc_op::Fin)))
-      new_cond = acc_cond((*this));
-    return disj.size() == 2;
-  }
-
-  bool
-  acc_cond::has_parity_prefix(acc_cond& new_cond,
-                              std::vector<unsigned>& colors) const
-  {
-    return code_.has_parity_prefix(new_cond, colors);
-  }
-
-  bool
-  acc_cond::is_parity_max_equiv(std::vector<int>&permut, bool even) const
-  {
-    if (code_.used_once_sets() != code_.used_sets())
-      return false;
-    bool result = code_.is_parity_max_equiv(permut, 0, even);
-    int max_value = *std::max_element(std::begin(permut), std::end(permut));
-    for (unsigned i = 0; i < permut.size(); ++i)
-      if (permut[i] != -1)
-        permut[i] = max_value - permut[i];
-      else
-        permut[i] = i;
-    return result;
-  }
-
   bool acc_cond::is_parity(bool& max, bool& odd, bool equiv) const
   {
     unsigned sets = num_;
@@ -1408,111 +1333,6 @@ namespace spot
     return patterns;
   }
 
-  bool
-  acc_cond::acc_code::is_parity_max_equiv(std::vector<int>& permut,
-                                          unsigned new_color,
-                                          bool even) const
-  {
-    auto conj = top_conjuncts();
-    auto disj = top_disjuncts();
-    if (conj.size() == 1)
-      {
-        if (disj.size() == 1)
-          {
-            acc_cond::acc_code elem = conj[0];
-            if ((even && elem.back().sub.op == acc_cond::acc_op::Inf)
-                || (!even && elem.back().sub.op == acc_cond::acc_op::Fin))
-              {
-                for (auto color : disj[0][0].mark.sets())
-                  {
-                    if (permut[color] != -1
-                        && ((unsigned) permut[color]) != new_color)
-                      return false;
-                    permut[color] = new_color;
-                  }
-                return true;
-              }
-            return false;
-          }
-        else
-          {
-            std::sort(disj.begin(), disj.end(),
-                      [](acc_code c1, acc_code c2)
-                      {
-                        return (c1 != c2) &&
-                          c1.back().sub.op == acc_cond::acc_op::Inf;
-                      });
-            unsigned i = 0;
-            for (; i < disj.size() - 1; ++i)
-              {
-                if (disj[i].back().sub.op != acc_cond::acc_op::Inf
-                    || !disj[i][0].mark.is_singleton())
-                  return false;
-                for (auto color : disj[i][0].mark.sets())
-                  {
-                    if (permut[color] != -1
-                        && ((unsigned) permut[color]) != new_color)
-                      return false;
-                    permut[color] = new_color;
-                  }
-              }
-            if (disj[i].back().sub.op == acc_cond::acc_op::Inf)
-              {
-                if (!even || !disj[i][0].mark.is_singleton())
-                  return false;
-                for (auto color : disj[i][0].mark.sets())
-                  {
-                    if (permut[color] != -1
-                        && ((unsigned) permut[color]) != new_color)
-                      return false;
-                    permut[color] = new_color;
-                  }
-                return true;
-              }
-            return disj[i].is_parity_max_equiv(permut, new_color + 1, even);
-          }
-      }
-    else
-      {
-        std::sort(conj.begin(), conj.end(),
-                  [](acc_code c1, acc_code c2)
-                  {
-                    return (c1 != c2)
-                      && c1.back().sub.op == acc_cond::acc_op::Fin;
-                  });
-        unsigned i = 0;
-        for (; i < conj.size() - 1; i++)
-          {
-            if (conj[i].back().sub.op != acc_cond::acc_op::Fin
-                || !conj[i][0].mark.is_singleton())
-              return false;
-            for (auto color : conj[i][0].mark.sets())
-              {
-                if (permut[color] != -1 && permut[color != new_color])
-                  return false;
-                permut[color] = new_color;
-              }
-          }
-        if (conj[i].back().sub.op == acc_cond::acc_op::Fin)
-          {
-            if (even)
-              return 0;
-            if (!conj[i][0].mark.is_singleton())
-              return false;
-            for (auto color : conj[i][0].mark.sets())
-              {
-                if (permut[color] != -1 && permut[color != new_color])
-                  return false;
-                permut[color] = new_color;
-              }
-            return true;
-          }
-
-        return conj[i].is_parity_max_equiv(permut, new_color + 1, even);
-      }
-  }
-
-
   namespace
   {
     template<typename Fun>
