@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2020, 2021 Laboratoire de Recherche et
+// Copyright (C) 2020-2023 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 //
 // This file is part of Spot, a model checking library.
@@ -29,6 +29,7 @@
 #include <spot/twaalgos/sbacc.hh>
 #include <spot/twaalgos/synthesis.hh>
 #include <spot/twaalgos/translate.hh>
+#include <spot/twaalgos/zlktree.hh>
 #include <spot/misc/minato.hh>
 #include <spot/twaalgos/totgba.hh>
 #include <spot/twaalgos/toparity.hh>
@@ -136,12 +137,12 @@ namespace{
   // Note, this only deals with deterministic strategies
   // Note, assumes that env starts playing
   twa_graph_ptr
-  apply_strategy(const twa_graph_ptr& arena,
+  apply_strategy(const const_twa_graph_ptr& arena,
                  bool unsplit, bool keep_acc)
   {
-    const auto& win = get_state_winners(arena);
-    const auto& strat = get_strategy(arena);
-    const auto& sp = get_state_players(arena);
+    const region_t& win = get_state_winners(arena);
+    const strategy_t& strat = get_strategy(arena);
+    const region_t& sp = get_state_players(arena);
     auto outs = get_synthesis_outputs(arena);
 
     if (!win[arena->get_init_state_number()])
@@ -446,9 +447,13 @@ namespace spot
   split_2step(const const_twa_graph_ptr& aut,
               const bdd& output_bdd, bool complete_env)
   {
+    assert(!aut->get_named_prop<region_t>("state-player")
+           && "aut is already split!");
     auto split = make_twa_graph(aut->get_dict());
 
     auto [has_unsat, unsat_mark] = aut->acc().unsat_mark();
+    bool max_par, odd_par, color_env;
+    color_env = aut->acc().is_parity(max_par, odd_par, true);
 
     split->copy_ap_of(aut);
     split->new_states(aut->num_states());
@@ -456,6 +461,7 @@ namespace spot
     set_synthesis_outputs(split, output_bdd);
 
     const auto use_color = has_unsat;
+    color_env &= use_color;
     if (has_unsat)
       split->copy_acceptance_of(aut);
     else
@@ -489,8 +495,10 @@ namespace spot
     // So we can first loop over the aut
     // and then deduce the owner
 
-    // a sort of hash-map for all new intermediate states
-    std::unordered_multimap<size_t, unsigned> env_hash;
+    // a sort of hash-map for all new intermediate stat
+    // second is the color of the incoming env trans
+    std::unordered_multimap<size_t,
+                            std::pair<unsigned, acc_cond::mark_t>> env_hash;
     env_hash.reserve((int) (1.5 * aut->num_states()));
     // a local map for edges leaving the current src
     // this avoids creating and then combining edges for each minterm
@@ -566,9 +574,7 @@ namespace spot
               // implies is faster than and
               if (bdd_implies(one_letter, e_info.einsup.first))
                 {
-                  e_info.econdout =
-                      bdd_appex(e_info.econd, one_letter,
-                                bddop_and, input_bdd);
+                  e_info.econdout = bdd_restrict(e_info.econd, one_letter);
                   dests.push_back(&e_info);
                   assert(e_info.econdout != bddfalse);
                 }
@@ -589,7 +595,7 @@ namespace spot
             auto range_h = env_hash.equal_range(h);
             for (auto it_h = range_h.first; it_h != range_h.second; ++it_h)
               {
-                unsigned i = it_h->second;
+                const auto& [i, this_color] = it_h->second;
                 auto out = split->out(i);
                 if (std::equal(out.begin(), out.end(),
                                dests.begin(), dests.end(),
@@ -611,9 +617,10 @@ namespace spot
                     if (it != env_edge_hash.end())
                       it->second.second |= one_letter;
                     else
-                      // Uncolored
                       env_edge_hash.emplace(i,
-                          eeh_t(split->new_edge(src, i, bddtrue), one_letter));
+                        eeh_t(split->new_edge(src, i, bddtrue,
+                                              this_color),
+                              one_letter));
                     break;
                   }
               }
@@ -621,12 +628,31 @@ namespace spot
             if (to_add)
               {
                 unsigned d = split->new_state();
-                unsigned n_e = split->new_edge(src, d, bddtrue);
-                env_hash.emplace(h, d);
+                auto this_color = acc_cond::mark_t({});
+                bool has_uncolored = false;
+                 for (const auto &t: dests)
+                  {
+                    split->new_edge(d, t->dst, t->econdout,
+                                    use_color ? t->acc
+                                              : acc_cond::mark_t({}));
+                    this_color |= t->acc;
+                    has_uncolored |= !t->acc;
+                  }
+
+                if (!color_env | has_uncolored)
+                  this_color = acc_cond::mark_t({});
+                else if (max_par)
+                  this_color =
+                    acc_cond::mark_t({this_color.min_set()-1});
+                else // min_par
+                  this_color =
+                    acc_cond::mark_t({this_color.max_set()-1});
+
+                unsigned n_e = split->new_edge(src, d, bddtrue, this_color);
+                env_hash.emplace(std::piecewise_construct,
+                                 std::forward_as_tuple(h),
+                                 std::forward_as_tuple(d, this_color));
                 env_edge_hash.emplace(d, eeh_t(n_e, one_letter));
-                for (const auto &t: dests)
-                  split->new_edge(d, t->dst, t->econdout,
-                                  use_color ? t->acc : acc_cond::mark_t({}));
               }
           } // letters
         // save locally stored condition
@@ -741,6 +767,9 @@ namespace spot
       case (algo::LAR_OLD):
         name = "lar.old";
         break;
+      case (algo::ACD):
+        name = "acd";
+        break;
     }
     return os << name;
 }
@@ -768,17 +797,15 @@ namespace spot
       auto sol = gi.s;
       const bdd_dict_ptr& dict = gi.dict;
 
-      for (auto&& p : std::vector<std::pair<const char*, int>>
-                        {{"simul", 0},
-                         {"ba-simul", 0},
-                         {"det-simul", 0},
-                         {"tls-impl", 1},
-                         {"wdba-minimize", 2}})
-        extra_options.set(p.first, extra_options.get(p.first, p.second));
+      extra_options.set_if_unset("simul", 0);
+      extra_options.set_if_unset("tls-impl", 1);
+      extra_options.set_if_unset("wdba-minimize", 2);
 
       translator trans(dict, &extra_options);
       switch (sol)
       {
+      case algo::ACD:
+        SPOT_FALLTHROUGH;
       case algo::LAR:
         SPOT_FALLTHROUGH;
       case algo::LAR_OLD:
@@ -808,13 +835,10 @@ namespace spot
       if (force_sbacc)
         dpa = sbacc(dpa);
       reduce_parity_here(dpa, true);
-      change_parity_here(dpa, parity_kind_max,
-                               parity_style_odd);
       assert((
                  [&dpa]() -> bool {
                    bool max, odd;
-                   dpa->acc().is_parity(max, odd);
-                   return max && odd;
+                   return dpa->acc().is_parity(max, odd);
                  }()));
       assert(is_deterministic(dpa));
       return dpa;
@@ -907,7 +931,6 @@ namespace spot
         if (bv)
           sw.start();
         dpa = split_2step(tmp, outs, true);
-        colorize_parity_here(dpa, true);
         if (bv)
           bv->split_time += sw.stop();
         if (vs)
@@ -930,7 +953,6 @@ namespace spot
         if (bv)
           sw.start();
         dpa = split_2step(aut, outs, true);
-        colorize_parity_here(dpa, true);
         if (bv)
           bv->split_time += sw.stop();
         if (vs)
@@ -956,6 +978,10 @@ namespace spot
           *vs << "determinization done\nDPA has "
               << dpa->num_states() << " states, "
               << dpa->num_sets() << " colors\n";
+        // The named property "state-player" is set in split_2step
+        // but not propagated by ntgba2dpa
+        alternate_players(dpa);
+        // Merge states knows about players
         dpa->merge_states();
         if (bv)
           bv->paritize_time += sw.stop();
@@ -964,11 +990,10 @@ namespace spot
               << dpa->num_states() << " states\n"
               << "determinization and simplification took "
               << bv->paritize_time << " seconds\n";
-        // The named property "state-player" is set in split_2step
-        // but not propagated by ntgba2dpa
-        alternate_players(dpa);
         break;
       }
+      case algo::ACD:
+        SPOT_FALLTHROUGH;
       case algo::LAR:
         SPOT_FALLTHROUGH;
       case algo::LAR_OLD:
@@ -978,20 +1003,20 @@ namespace spot
         if (gi.s == algo::LAR)
           {
             dpa = to_parity(aut);
-            // reduce_parity is called by to_parity(),
-            // but with colorization turned off.
-            colorize_parity_here(dpa, true);
+            reduce_parity_here(dpa, false);
           }
-        else
+        else if (gi.s == algo::LAR_OLD)
           {
             dpa = to_parity_old(aut);
-            dpa = reduce_parity_here(dpa, true);
+            reduce_parity_here(dpa, true);
           }
-        change_parity_here(dpa, parity_kind_max, parity_style_odd);
+        else
+          dpa = acd_transform(aut);
         if (bv)
           bv->paritize_time += sw.stop();
         if (vs)
-          *vs << "LAR construction done in " << bv->paritize_time
+          *vs << (gi.s == algo::ACD ? "ACD" : "LAR")
+              << " construction done in " << bv->paritize_time
               << " seconds\nDPA has "
               << dpa->num_states() << " states, "
               << dpa->num_sets() << " colors\n";
@@ -999,7 +1024,6 @@ namespace spot
         if (bv)
           sw.start();
         dpa = split_2step(dpa, outs, true);
-        colorize_parity_here(dpa, true);
         if (bv)
           bv->split_time += sw.stop();
         if (vs)
@@ -1040,8 +1064,10 @@ namespace spot
   }
 
   twa_graph_ptr
-  solved_game_to_separated_mealy(twa_graph_ptr arena, synthesis_info& gi)
+  solved_game_to_mealy(twa_graph_ptr arena, synthesis_info& gi)
   {
+    // This currently always produces either separated or split mealy machines
+    // if this changes, change also the other functions accordingly
     assert(arena->get_named_prop<region_t>("state-player")
            && "Need prop \"state-player\"");
     if (!arena)
@@ -1055,27 +1081,45 @@ namespace spot
     if (!get_state_winner(arena, arena->get_init_state_number()))
       return nullptr;
 
-    // If we use minimizations 0,1 or 2 -> unsplit
-    const bool do_unsplit = gi.minimize_lvl < 3;
-    auto m = apply_strategy(arena, do_unsplit, false);
+    auto m = apply_strategy(arena, false, false);
 
     m->prop_universal(true);
 
-    if ((0 < gi.minimize_lvl) && (gi.minimize_lvl < 3))
-      reduce_mealy_here(m, gi.minimize_lvl == 2);
-    else if (gi.minimize_lvl >= 3)
-      m = minimize_mealy(m, gi.minimize_lvl - 4);
-
-    if (!do_unsplit)
-      m = unsplit_mealy(m);
-
     if (gi.bv)
       {
+        auto sp = get_state_players(m);
+        auto n_s_env = sp.size() - std::accumulate(sp.begin(),
+                                                   sp.end(),
+                                                   0u);
+        auto n_e_env = 0u;
+        std::for_each(m->edges().begin(), m->edges().end(),
+                      [&n_e_env, &sp](const auto& e)
+                        {
+                          n_e_env += sp[e.src];
+                        });
         gi.bv->strat2aut_time += sw.stop();
-        gi.bv->nb_strat_states += m->num_states();
-        gi.bv->nb_strat_edges += m->num_edges();
+        gi.bv->nb_strat_states += n_s_env;
+        gi.bv->nb_strat_edges += n_e_env;
       }
 
+    assert(is_mealy(m));
+    return m;
+  }
+
+  twa_graph_ptr
+  solved_game_to_mealy(twa_graph_ptr arena)
+  {
+    synthesis_info dummy;
+    return solved_game_to_mealy(arena, dummy);
+  }
+
+  twa_graph_ptr
+  solved_game_to_separated_mealy(twa_graph_ptr arena, synthesis_info& gi)
+  {
+    auto m = solved_game_to_mealy(arena, gi);
+
+    if (m->get_named_prop<region_t>("state-player"))
+      m = unsplit_mealy(m);
     assert(is_separated_mealy(m));
     return m;
   }
@@ -1087,6 +1131,27 @@ namespace spot
     return solved_game_to_separated_mealy(arena, dummy);
   }
 
+  twa_graph_ptr
+  solved_game_to_split_mealy(twa_graph_ptr arena, synthesis_info& gi)
+  {
+    auto m = solved_game_to_mealy(arena, gi);
+
+    if (!m->get_named_prop<region_t>("state-player"))
+      {
+        assert(is_separated_mealy(m));
+        split_separated_mealy_here(m);
+      }
+    assert(is_split_mealy(m));
+    return m;
+  }
+
+  twa_graph_ptr
+  solved_game_to_split_mealy(twa_graph_ptr arena)
+  {
+    synthesis_info dummy;
+    return solved_game_to_split_mealy(arena, dummy);
+  }
+
 }
 
 namespace spot
@@ -1094,6 +1159,27 @@ namespace spot
   //Anonymous for try_create_strat
   namespace
   {
+    // Checks that 2 sets have a common element. Use it instead
+    // of set_intersection when we just want to check if they have a common
+    // element because it avoids going through the rest of the sets after an
+    // element is found.
+    static bool
+    are_intersecting(const std::set<formula> &v1,
+                    const std::set<formula> &v2)
+    {
+      auto v1_pos = v1.begin(), v2_pos = v2.begin(), v1_end = v1.end(),
+          v2_end = v2.end();
+      while (v1_pos != v1_end && v2_pos != v2_end)
+        {
+          if (*v1_pos < *v2_pos)
+            ++v1_pos;
+          else if (*v2_pos < *v1_pos)
+            ++v2_pos;
+          else
+            return true;
+        }
+      return false;
+    }
     class formula_2_inout_props
     {
     private:
@@ -1135,16 +1221,19 @@ namespace spot
   mealy_like
   try_create_direct_strategy(formula f,
                              const std::vector<std::string>& output_aps,
-                             synthesis_info &gi)
+                             synthesis_info &gi, bool want_strategy)
   {
     auto vs = gi.verbose_stream;
     auto& bv = gi.bv;
+    bdd_dict_ptr& dict = gi.dict;
+    int tmp;
 
     if (vs)
       *vs << "trying to create strategy directly for " << f << '\n';
 
-    auto ret_sol_maybe = [&vs]()
+    auto ret_sol_maybe = [&vs, &tmp, &dict]()
       {
+        dict->unregister_all_my_variables(&tmp);
         if (vs)
           *vs << "direct strategy might exist but was not found.\n";
         return mealy_like{
@@ -1152,8 +1241,9 @@ namespace spot
                   nullptr,
                   bddfalse};
       };
-    auto ret_sol_none = [&vs]()
+    auto ret_sol_none = [&vs, &tmp, &dict]()
       {
+        dict->unregister_all_my_variables(&tmp);
         if (vs)
           *vs << "no strategy exists.\n";
         return mealy_like{
@@ -1162,14 +1252,23 @@ namespace spot
                   bddfalse};
       };
 
-    auto ret_sol_exists = [&vs](auto strat)
+    auto ret_sol_exists =
+      [&vs, &want_strategy, &tmp, &dict](twa_graph_ptr strat)
       {
+        dict->unregister_all_my_variables(&tmp);
         if (vs)
           {
-            *vs << "direct strategy was found.\n"
-                << "direct strat has " << strat->num_states()
-                << " states and " << strat->num_sets() << " colors\n";
+            *vs << "direct strategy was found.\n";
+            if (want_strategy)
+            {
+                *vs << "direct strat has " << strat->num_states()
+                << " states, " << strat->num_edges()
+                << " edges and " << strat->num_sets() << " colors\n";
+
+            }
           }
+        if (strat)
+          strat->merge_edges();
         return mealy_like{
                   mealy_like::realizability_code::REALIZABLE_REGULAR,
                   strat,
@@ -1177,88 +1276,146 @@ namespace spot
       };
     formula_2_inout_props form2props(output_aps);
 
-    auto output_aps_set = std::set<std::string>(output_aps.begin(),
-                                                output_aps.end());
-
-    formula f_g = formula::tt(), f_left, f_right;
-
-    // If we have a formula like G(b₁) ∧ (φ ↔ GFb₂), we extract b₁ and
-    // continue the construction for (φ ↔ GFb₂).
+    formula f_g, f_other;
+    // If it is G(α) ∧ G(β) ∧ …
     if (f.is(op::And))
     {
-      if (f.size() != 2)
-        return ret_sol_maybe();
-      if (f[0].is(op::G) && f[0][0].is_boolean())
-      {
-        f_g = f[0];
-        f = f[1];
-      }
-      else if (f[1].is(op::G) && f[1][0].is_boolean())
-      {
-        f_g = f[1];
-        f = f[0];
-      }
-      else
-        return ret_sol_maybe();
-    }
-    if (f.is(op::Equiv))
-    {
-      auto [left_ins, left_outs] = form2props.aps_of(f[0]);
-      auto [right_ins, right_outs] = form2props.aps_of(f[1]);
+      std::vector<formula> gs;
+      std::vector<formula> others;
+      for (auto child : f)
+        if (child.is(op::G) && child[0].is_boolean())
+          gs.push_back(child[0]);
+        else
+          others.push_back(child);
 
-      auto properties_vector = [](const formula& f,
-                                  const std::set<formula>& ins,
-                                  const std::set<formula>& outs)
+      f_g = formula::And(gs);
+      f_other = formula::And(others);
+    }
+    else if (f.is(op::G) && f[0].is_boolean())
+    {
+      f_g = f[0];
+      f_other = formula::tt();
+    }
+    else
+    {
+      f_g = formula::tt();
+      f_other = f;
+    }
+
+    // We have to check if the content of G is realizable (input-complete)
+    bdd output_bdd_tmp = bddtrue;
+    for (auto& out : output_aps)
+      output_bdd_tmp &= bdd_ithvar(
+        dict->register_proposition(formula::ap(out), &tmp));
+
+    if (!f_g.is_tt())
+    {
+      auto g_bdd = formula_to_bdd(f_g, dict, &tmp);
+      if (bdd_exist(g_bdd, output_bdd_tmp) != bddtrue)
+        return ret_sol_none();
+    }
+
+    if (f_other.is(op::Equiv))
+    {
+      // Check if FG or GF
+      auto is_general = [&tmp, &output_bdd_tmp, &dict](const formula &f,
+                                                       op first, op second)
       {
-        return std::vector<bool>
-        {
-          f.is({op::G, op::F}) && f[0][0].is_boolean() && ins.empty(),
-          f.is_syntactic_recurrence() && outs.empty(),
-          // f is FG(bool)
-          f.is({op::F, op::G}) && f[0][0].is_boolean() && ins.empty(),
-          f.is_syntactic_persistence() && outs.empty()
-        };
+        if (!f.is({first, second}) || !f[0][0].is_boolean())
+          return false;
+        auto f_bdd = formula_to_bdd(f[0][0], dict, &tmp);
+        if (bdd_exist(f_bdd, output_bdd_tmp) != bddtrue)
+          return false;
+        f_bdd = formula_to_bdd(formula::Not(f[0][0]), dict, &tmp);
+        bool res = (bdd_exist(f_bdd, output_bdd_tmp) == bddtrue);
+        return res;
       };
-      // We need to detect
-      // GF(outs) ↔ recurrence(ins),
-      // recurrence(ins) ↔ GF(outs),
-      // FG(outs) ↔ persistence(ins),
-      // persistence(ins) ↔ FG(outs)
-      const auto left_properties = properties_vector(f[0], left_ins, left_outs),
-              right_properties = properties_vector(f[1], right_ins, right_outs);
+
+      auto is_gf = [is_general](const formula& f)
+      {
+        return is_general(f, op::G, op::F);
+      };
+
+      auto is_fg = [is_general](const formula& f)
+      {
+        return is_general(f, op::F, op::G);
+      };
+
+      auto is_co_bu = [](const formula &f, const std::set<formula>& outs)
+      {
+        return outs.empty() && f.is_syntactic_obligation();
+      };
+
+      auto is_buchi = [](const formula &f, const std::set<formula>& outs)
+      {
+        return outs.empty() && f.is_syntactic_recurrence();
+      };
+
+      auto properties_vector = [&](const formula &f,
+                            const std::set<formula> &outs)
+      {
+        auto is_lgf = is_gf(f);
+        auto is_lfg = is_fg(f);
+        return std::vector<bool>{
+            // f is GF(ins + outs) <-> buchi(ins)
+            is_lgf,
+            is_buchi(f, outs),
+            // f is FG(ins + outs) <-> co-buchi(ins)
+            is_lfg,
+            is_co_bu(f, outs)};
+      };
+
+
+      auto [left_ins, left_outs] = form2props.aps_of(f_other[0]);
+      auto [right_ins, right_outs] = form2props.aps_of(f_other[1]);
+
+      auto left_properties = properties_vector(f_other[0], left_outs);
+      auto right_properties = properties_vector(f_other[1], right_outs);
+
       unsigned combin = -1U;
       for (unsigned i = 0; i < 4; ++i)
-      {
-        if (left_properties[i] && right_properties[(i%2) ? (i-1) : (i+1)])
+        if (left_properties[i] && right_properties[(i % 2) ? (i - 1) : (i + 1)])
         {
           combin = i;
           break;
         }
-      }
+
+      // If we don't match, we don't know
       if (combin == -1U)
         return ret_sol_maybe();
 
-      // left is the recurrence (resp. persistence)
-      // right is GF(outs) (resp. GF(outs))
-      // If f[0] is GF or FG
-      f_left = f[(combin+1)%2];
-      f_right = f[combin%2];
-      if (!(combin%2))
+      formula f_left = f_other[(combin + 1) % 2];
+      formula f_right = f_other[combin % 2];
+      if (!(combin % 2))
       {
         std::swap(left_ins, right_ins);
         std::swap(left_outs, right_outs);
       }
 
+      auto [_, g_outs] = form2props.aps_of(f_g);
+      if (are_intersecting(g_outs, right_outs))
+        return ret_sol_maybe();
+
+      // We know that a strategy exists and we don't want to construct it.
+      if (!want_strategy)
+        return ret_sol_exists(nullptr);
+
       auto trans = create_translator(gi);
-      trans.set_type(combin < 2 ? postprocessor::Buchi
-                                : postprocessor::CoBuchi);
+
       trans.set_pref(postprocessor::Deterministic | postprocessor::Complete);
+      if (combin < 2)
+        trans.set_type(postprocessor::Buchi);
+      else
+        trans.set_type(postprocessor::CoBuchi);
 
       stopwatch sw;
       if (bv)
         sw.start();
       auto res = trans.run(f_left);
 
+      if (!is_deterministic(res))
+        return ret_sol_maybe();
+
       if (bv)
       {
         auto delta = sw.stop();
@@ -1266,79 +1423,76 @@ namespace spot
         if (vs)
           *vs << "tanslating formula done in " << delta << " seconds\n";
       }
-
-      if (!is_deterministic(res))
-        return ret_sol_maybe();
-      for (auto& out : right_outs)
-          res->register_ap(out.ap_name());
-
-      // The BDD that describes the content of the G in a conjunction
-      bdd g_bdd = bddtrue;
-
-      // Convert the set of outputs to a BDD
-      bdd output_bdd = bddtrue;
-      for (auto &out : output_aps_set)
-        output_bdd &= bdd_ithvar(res->register_ap(out));
-
-      if (!f_g.is_tt())
-      {
-        g_bdd = formula_to_bdd(f_g[0], res->get_dict(), res);
-        // If the content of G is not input-complete, a simple strategy for
-        // env is to play this missing value.
-        if (bdd_exist(g_bdd, output_bdd) != bddtrue)
-        {
-          return ret_sol_none();
-        }
-      }
-
-      // For the GF(outs) (resp. GF(outs)), the content and its negation can be
-      // converted to a BDD.
-      bdd right_bdd, neg_right_bdd;
-      if (combin < 2)
-      {
-        right_bdd = formula_to_bdd(f_right[0][0], res->get_dict(), res);
-        neg_right_bdd = bdd_not(right_bdd);
-      }
-      else
-      {
-        neg_right_bdd = formula_to_bdd(f_right[0][0], res->get_dict(), res);
-        right_bdd = bdd_not(neg_right_bdd);
-      }
-      // Monitor is a special case. As we color accepting transitions, if the
-      // acceptance is true, we cannot say that a transition is accepting if
-      // a color is seen.
-      const bool is_true = res->acc().is_t();
-      scc_info si(res, scc_info_options::NONE);
-      for (auto& e : res->edges())
-      {
-        // Here the part describing the outputs is based on the fact that
-        // they must be seen infinitely often. As these edges are seen
-        // finitely often, we can let the minimization choose the value.
-        if (si.scc_of(e.src) == si.scc_of(e.dst))
-          {
-            if (e.acc || is_true)
-              e.cond &= right_bdd;
-            else
-              e.cond &= neg_right_bdd;
-          }
-        // g_bdd has to be true all the time. So we cannot only do it
-        // between SCCs.
-        e.cond &= g_bdd;
-        if (e.cond == bddfalse)
-          return ret_sol_maybe();
-        // The recurrence is Büchi but the strategy is a monitor. We need
-        // to remove the color.
-        e.acc = {};
-      }
-
-      set_synthesis_outputs(res, output_bdd);
-      res->set_acceptance(acc_cond::acc_code::t());
-
       res->prop_complete(trival::maybe());
+
+      bdd output_bdd = bddtrue;
+      auto [is, os] = form2props.aps_of(f);
+      for (auto i : is)
+        res->register_ap(i);
+      for (auto o : os)
+        output_bdd &= bdd_ithvar(res->register_ap(o));
+
+      bdd right_bdd = formula_to_bdd(f_right[0][0], dict, res);
+      bdd neg_right_bdd = bdd_not(right_bdd);
+      bdd g_bdd = formula_to_bdd(f_g, dict, res);
+
+      if (combin > 1)
+        std::swap(right_bdd, neg_right_bdd);
+
+      right_bdd = bdd_and(right_bdd, g_bdd);
+      neg_right_bdd = bdd_and(neg_right_bdd, g_bdd);
+
+      scc_info si(res, scc_info_options::NONE);
+
+      bool is_true_acc = ((combin < 2) && res->acc().is_t())
+                || ((combin > 1) && res->acc().is_f());
+      auto prop_vector = propagate_marks_vector(res);
+      auto& ev = res->edge_vector();
+      for (unsigned i = 1; i < ev.size(); ++i)
+      {
+        auto &edge = ev[i];
+        if (si.scc_of(edge.src) == si.scc_of(edge.dst))
+        {
+          if (edge.acc || is_true_acc)
+            edge.cond &= right_bdd;
+          // If we have a GF and an edge is not colored but prop_vector says
+          // that this edge could be colored, it means that we can do what we
+          // want
+          else if (!prop_vector[i])
+            edge.cond &= neg_right_bdd;
+          else
+            edge.cond &= g_bdd;
+        }
+        else
+          edge.cond &= g_bdd;
+        edge.acc = {};
+      }
+      res->set_acceptance(acc_cond::acc_code::t());
+      res->set_named_prop<bdd>("synthesis-outputs", new bdd(output_bdd));
+
       return ret_sol_exists(res);
     }
-    else
-      return ret_sol_maybe();
+    else if (f_other.is_tt())
+    {
+      if (!want_strategy)
+        return ret_sol_exists(nullptr);
+      auto res = make_twa_graph(dict);
+
+      bdd output_bdd = bddtrue;
+      auto [ins_f, _] = form2props.aps_of(f_g);
+      for (auto &out : output_aps)
+        output_bdd &= bdd_ithvar(res->register_ap(out));
+
+      for (auto &in : ins_f)
+        res->register_ap(in);
+
+      res->set_named_prop<bdd>("synthesis-outputs", new bdd(output_bdd));
+      bdd g_bdd = formula_to_bdd(f_g, dict, res);
+      res->new_state();
+      res->new_edge(0, 0, g_bdd);
+      return ret_sol_exists(res);
+    }
+    return ret_sol_maybe();
   }
 
 } // spot
@@ -1346,28 +1500,6 @@ namespace spot
 namespace // anonymous for subsformula
 {
   using namespace spot;
-  // Checks that 2 sets have a common element. Use it instead
-  // of set_intersection when we just want to check if they have a common
-  // element because it avoids going through the rest of the sets after an
-  // element is found.
-  static bool
-  are_intersecting(const std::set<formula> &v1,
-                   const std::set<formula> &v2)
-  {
-    auto v1_pos = v1.begin(), v2_pos = v2.begin(), v1_end = v1.end(),
-         v2_end = v2.end();
-    while (v1_pos != v1_end && v2_pos != v2_end)
-      {
-        if (*v1_pos < *v2_pos)
-          ++v1_pos;
-        else if (*v2_pos < *v1_pos)
-          ++v2_pos;
-        else
-          return true;
-      }
-    return false;
-  }
-
   static std::pair<std::set<formula>, std::set<formula>>
   algo4(const std::vector<formula> &assumptions,
         const std::set<std::string> &outs,
@@ -1603,7 +1735,7 @@ namespace // anonymous for subsformula
         std::vector<formula> children;
         for (auto fi : f)
           children.push_back(
-            extract_and(fi, outs, can_extract_impl, form2props));
+            extract_and(fi, outs, false, form2props));
         return formula::And(children);
       }
     if (f.is(op::Not))
