@@ -1,5 +1,5 @@
 // -*- coding: utf-8 -*-
-// Copyright (C) 2008-2012, 2014-2019 Laboratoire de Recherche et
+// Copyright (C) 2008-2012, 2014-2019, 2022 Laboratoire de Recherche et
 // Développement de l'Epita (LRDE).
 // Copyright (C) 2004, 2005 Laboratoire d'Informatique de Paris
 // 6 (LIP6), département Systèmes Répartis Coopératifs (SRC),
@@ -69,9 +69,11 @@ const char* default_algos[] = {
   "Cou99abs",
   "CVWY90",
   "CVWY90(bsh=4K)",
+  "CVWY90(ar:from_stack)",
   "GV04",
   "SE05",
   "SE05(bsh=4K)",
+  "SE05(ar:from_stack)",
   "Tau03",
   "Tau03_opt",
   "Tau03_opt(condstack)",
