@@ -1,5 +1,5 @@
 # -*- mode: python; coding: utf-8 -*-
-# Copyright (C) 2019  Laboratoire de Recherche et Développement
+# Copyright (C) 2019, 2022 Laboratoire de Recherche et Développement
 # de l'Epita
 #
 # This file is part of Spot, a model checking library.
@@ -18,16 +18,18 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import spot
+from unittest import TestCase
+tc = TestCase()
 
 
 aut = spot.translate('a U (c & Gb)')
-assert not spot.is_terminal_automaton(aut)
-assert aut.prop_terminal().is_false()
+tc.assertFalse(spot.is_terminal_automaton(aut))
+tc.assertTrue(aut.prop_terminal().is_false())
 rem = spot.remove_ap()
 rem.add_ap("b")
 aut = rem.strip(aut)
-assert not aut.prop_terminal().is_false()
-assert spot.is_terminal_automaton(aut)
-assert aut.prop_terminal().is_true()
+tc.assertFalse(aut.prop_terminal().is_false())
+tc.assertTrue(spot.is_terminal_automaton(aut))
+tc.assertTrue(aut.prop_terminal().is_true())
 aut = rem.strip(aut)
-assert aut.prop_terminal().is_true()
+tc.assertTrue(aut.prop_terminal().is_true())
